/*****************************************************************************
 * Copyright (c) 2011 CEA LIST.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *		
 *		CEA LIST - Initial API and implementation
 *
 *****************************************************************************/
package org.eclipse.papyrus.gmf.diagram.common.edit.policy;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FigureListener;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.editpolicies.SelectionEditPolicy;
import org.eclipse.gmf.runtime.draw2d.ui.figures.WrappingLabel;

/**
 * Non diagram-specific class generated by GMF Tooling.
 */
public class TextSelectionEditPolicy extends SelectionEditPolicy {

	private IFigure selectionFeedbackFigure;

	private IFigure focusFeedbackFigure;

	private FigureListener hostPositionListener;

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void showPrimarySelection() {
		if(getHostFigure() instanceof WrappingLabel) {
			((WrappingLabel)getHostFigure()).setSelected(true);
			((WrappingLabel)getHostFigure()).setFocus(true);
		} else {
			showSelection();
			showFocus();
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void showSelection() {
		if(getHostFigure() instanceof WrappingLabel) {
			((WrappingLabel)getHostFigure()).setSelected(true);
			((WrappingLabel)getHostFigure()).setFocus(false);
		} else {
			hideSelection();
			addFeedback(selectionFeedbackFigure = createSelectionFeedbackFigure());
			getHostFigure().addFigureListener(getHostPositionListener());
			refreshSelectionFeedback();
			hideFocus();
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void hideSelection() {
		if(getHostFigure() instanceof WrappingLabel) {
			((WrappingLabel)getHostFigure()).setSelected(false);
			((WrappingLabel)getHostFigure()).setFocus(false);
		} else {
			if(selectionFeedbackFigure != null) {
				removeFeedback(selectionFeedbackFigure);
				getHostFigure().removeFigureListener(getHostPositionListener());
				selectionFeedbackFigure = null;
			}
			hideFocus();
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void showFocus() {
		if(getHostFigure() instanceof WrappingLabel) {
			((WrappingLabel)getHostFigure()).setFocus(true);
		} else {
			hideFocus();
			addFeedback(focusFeedbackFigure = createFocusFeedbackFigure());
			refreshFocusFeedback();
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void hideFocus() {
		if(getHostFigure() instanceof WrappingLabel) {
			((WrappingLabel)getHostFigure()).setFocus(false);
		} else {
			if(focusFeedbackFigure != null) {
				removeFeedback(focusFeedbackFigure);
				focusFeedbackFigure = null;
			}
		}
	}

	protected Rectangle getFeedbackBounds() {
		Rectangle bounds;
		if(getHostFigure() instanceof Label) {
			bounds = ((Label)getHostFigure()).getTextBounds();
			bounds.intersect(getHostFigure().getBounds());
		} else {
			bounds = getHostFigure().getBounds().getCopy();
		}
		getHostFigure().getParent().translateToAbsolute(bounds);
		getFeedbackLayer().translateToRelative(bounds);
		return bounds;
	}

	protected IFigure createSelectionFeedbackFigure() {
		if(getHostFigure() instanceof Label) {
			Label feedbackFigure = new Label();
			feedbackFigure.setOpaque(true);
			feedbackFigure.setBackgroundColor(ColorConstants.menuBackgroundSelected);
			feedbackFigure.setForegroundColor(ColorConstants.menuForegroundSelected);
			return feedbackFigure;
		} else {
			RectangleFigure feedbackFigure = new RectangleFigure();
			feedbackFigure.setFill(false);
			return feedbackFigure;
		}
	}

	protected IFigure createFocusFeedbackFigure() {
		return new Figure() {

			protected void paintFigure(Graphics graphics) {
				graphics.drawFocus(getBounds().getResized(-1, -1));
			}
		};
	}

	protected void updateLabel(Label target) {
		Label source = (Label)getHostFigure();
		target.setText(source.getText());
		target.setTextAlignment(source.getTextAlignment());
		target.setFont(source.getFont());
	}

	protected void refreshSelectionFeedback() {
		if(selectionFeedbackFigure != null) {
			if(selectionFeedbackFigure instanceof Label) {
				updateLabel((Label)selectionFeedbackFigure);
				selectionFeedbackFigure.setBounds(getFeedbackBounds());
			} else {
				selectionFeedbackFigure.setBounds(getFeedbackBounds().expand(5, 5));
			}
		}
	}

	protected void refreshFocusFeedback() {
		if(focusFeedbackFigure != null) {
			focusFeedbackFigure.setBounds(getFeedbackBounds());
		}
	}

	public void refreshFeedback() {
		refreshSelectionFeedback();
		refreshFocusFeedback();
	}

	private FigureListener getHostPositionListener() {
		if(hostPositionListener == null) {
			hostPositionListener = new FigureListener() {

				public void figureMoved(IFigure source) {
					refreshFeedback();
				}
			};
		}
		return hostPositionListener;
	}
}
